import math
import os
UNDEFINED = "undefined"
import exceptions

class ConfigResourceMissing(exceptions.Exception):
	def __init__(self, arg):
            self.arg = arg


# calculate95Conf
#TODO: Handle division by zero
def calculate95Conf(p, n):
    if p != UNDEFINED and n != UNDEFINED and int(n) != 0:
        return 1.96*math.sqrt(p*(1-p)/float(n))
    else:
        return UNDEFINED

def filterListOnlyNumbers(lst):
    return filter(lambda x:x != UNDEFINED, lst)

def computeMeanForNumbersInList(lst):
    filtered = filterListOnlyNumbers(lst)
    if len(filtered) > 0:
        return float(sum(filtered))/len(filtered)
    else:
        return UNDEFINED

def computeMaxForNumbersInList(lst):
    filtered = filterListOnlyNumbers(lst)
    if len(filtered) > 0:
        return max(filterListOnlyNumbers(lst))
    else:
        return UNDEFINED

def computeMinForNumbersInList(lst):
    filtered = filterListOnlyNumbers(lst)
    if len(filtered) > 0:
        return min(filterListOnlyNumbers(lst))
    else:
        return UNDEFINED

def computeFScore(precision, recall):
    if precision != UNDEFINED and recall != UNDEFINED and (precision + recall) != 0:
        return 2*precision*recall/float(precision + recall)
    else:
        return UNDEFINED 

def computePrecision(correctFound, totalFound):
    return doDivision(correctFound, totalFound)

def computeRecall(correctFound, totalInGold):
    return doDivision(correctFound, totalInGold)

def doDivision(correct, total):
    if total > 0:
        return float(correct)/total
    else:
        return UNDEFINED

def ensure_dir(dir):
    if not os.path.exists(dir):
        os.makedirs(dir)

def ensure_file(path):
    if not os.path.exists(path):
        open(path, "w").close()

def getImport(importFolder, importFile):
    try:
        exec("import " + importFolder + "." + importFile + " as imp")
        return imp
    except ImportError, fel:
        print "Cannot find: " + importFolder + "/" + importFile + ".py"
        print type(fel)
        print fel
        raise fel

def raiseExceptionIfResourceDoesNotExist(resourceToCheck):
     if not os.path.exists(resourceToCheck):
         print "WARNING: The config resource " + resourceToCheck + " is missing\n"
         raise ConfigResourceMissing(resourceToCheck)
           
def latinDecode(str):
    try:
        return str.decode('ISO 8859-1')
    except:
        print "Could not decode " + str
        return ""
  
def getCodedWord(inStr, coding):
    lst = []
    for ch in inStr:
        try:
            encoded = ch.encode(coding)
	    lst.append(encoded)
        except UnicodeEncodeError:
            try:
                print "Failed to encode " + ch + ". Will be replaced by 'a'"
            except:
                 print "Failed to encode character. Will be replaced by 'a'"
            encoded ='a'.encode(coding)
            lst.append(encoded)
    return "".encode(coding).join(lst)
      
def getLatinCodedWord(inStr):
    return getCodedWord(inStr, 'ISO 8859-1')

def getUTFCodedWord(inStr):
    return getCodedWord(inStr, 'utf8')

def getUnicodedWordFromLatinCoded(inStr):
	return getUnicodedWordFromCoding(inStr,'ISO 8859-1')
        
def getUnicodedWordFromUTFCoded(inStr):
	return getUnicodedWordFromCoding(inStr, 'utf8')
  
def getUnicodedWordFromCoding(inStr, coding):
    lst = []
    #str = u''
    for ch in inStr.split(" "):
        try:
            encoded = ch.decode(coding)
            lst.append(encoded)
        except UnicodeDecodeError:
            try:
                print "Failed to decode " + ch + ". Will be replaced by 'a'"
            except:
                 print "Failed to decode character. Will be replaced by 'a'"
            encoded ='a'.decode(coding)
            lst.append(encoded)
    return  u' '.join(lst).strip()

def addBag(element, dictionary):
    if dictionary.has_key(element):
        dictionary[element] = dictionary[element] + 1
    else: 
        dictionary[element] = 1

#print computeMeanForNumbersInList([UNDEFINED, UNDEFINED, 1, 2, 0, 2, UNDEFINED])
#print computeMaxForNumbersInList([UNDEFINED, UNDEFINED, 1, 2, 0, 2, UNDEFINED])
#print computeMinForNumbersInList([UNDEFINED, UNDEFINED, 1, 2, 0, 2, UNDEFINED])

