# -*- coding: utf-8 -*-

import csv
import glob
import runCRFProperties
import common
import foldEvaluationConfig.configForSerie
import levenshtein
import permutations
import compoundSplitting

import sys
import time
sys.path.append("./foldEvaluationConfig")

NOMATCH = "O"
termDict = {}

#TODO: Kolla om dubbla förekomster blir rätt

#Always create a new dictionary
def createDictionary(combineCategories, name):
    termDict = {}
    #if termDict == {}:
    return doCreateDictionary(combineCategories, name)
    #else:
    #    return termDict

def doCreateDictionary(combineCategories, name):
    print "Creates new Snomed feature lexicons. The variable combineCategories = " + str(combineCategories)

    config = common.getImport(name, name + "_config")
    terminlogyFiles = config.terminlogyFiles

    for el in terminlogyFiles:
        globpath = foldEvaluationConfig.configForSerie.terminologyPath + el
        #print "Lexikon glob expression " + globpath
        files =  glob.glob(globpath)
        #print "Lexikonfiles to use " + str(files)
        for file in files:
            print file
            terminologyFile = open(file, "r")
            termCategory = common.getUnicodedWordFromUTFCoded(file)
            try:
                termCategory = common.getUnicodedWordFromUTFCoded(file.split("/")[-1].split(".txt")[0])
                #print "Using the following as TermCategory: " +  termCategory
            except:
                print "Using original filename as termCategory: " +  termCategory
            for line in terminologyFile:
                stripped = common.getUnicodedWordFromUTFCoded(line).strip().lower()
                withoutcomma = stripped.replace(",","")
                if termDict.has_key(stripped):
                    if combineCategories and not termCategory in termDict[stripped]:
                        termDict[stripped] = termDict[stripped] + "_" + termCategory
                else:
                    termDict[stripped] = termCategory
                if withoutcomma != stripped:
                   if termDict.has_key(withoutcomma):
                       if combineCategories and not termCategory in termDict[withoutcomma]:
                           termDict[withoutcomma] = termDict[withoutcomma] + "_" + termCategory
                   else:
                       termDict[withoutcomma] = termCategory
            terminologyFile.close()
    
    return termDict

    #totlexikon = open("totlexikon.txt", "w")
    #for key, item in termDict.items():
    #    totlexikon.write(key + " " + item + "\n")
    #totlexikon.close()

def getWords(featureList):
    words = []
    for line in featureList:
        words.append(line[0])
    return words

def findTermInMatrix(wordListWithFeatures, termDict, name, combineCategories):
    wordListLemma = [el[3] for el in wordListWithFeatures]
    wordListOrig = [el[0] for el in wordListWithFeatures]
    result = findTermInSequence(wordListOrig, wordListLemma, termDict, name, combineCategories)
    return result

def findTermInSequence(wordListOrig, wordListLemma, termDict, name, combineCategories):
    if len(wordListOrig) != len(wordListLemma):
        print "WARNING: NUMER OF LEMMATIZED TOKENS AND DOES NOT MATCH NON-LEMMATIZED"
    nrOfWords = len(wordListOrig)
    if nrOfWords < 1:
        return []
    for searchedTermLength in range(nrOfWords, 0, -1):
        found = findTermInSequenceWithLength(searchedTermLength, wordListOrig, wordListLemma, termDict, name, combineCategories)
        if found:
            #print "*********" + str(found)
            return found
    #print "inget hittat", wordList
    nothingFoundList = []
    for el in wordListOrig:
        nothingFoundList.append((el, NOMATCH)) 
    return nothingFoundList

def findTermInSequenceWithLength(searchedTermLength, wordListOrig, wordListLemma, termDict, name, combineCategories):
    config = common.getImport(name, name + "_config")
    for index in range(0, len(wordListOrig) - searchedTermLength + 1):
        wordstrOrig = " ".join(wordListOrig[index:index + searchedTermLength])
        wordstrLemma = " ".join(wordListLemma[index:index + searchedTermLength])
        if termDict.has_key(wordstrOrig.lower()):
            snomedNumber = termDict[wordstrOrig.lower()]
            return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, snomedNumber, name, combineCategories)
        elif termDict.has_key(wordstrLemma.lower()) and  config.doLemmatization:
            snomedNumber = termDict[wordstrLemma.lower()]
            return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, snomedNumber, name, combineCategories)
        elif config.doLevenshtein:
            ldOrig = getLevenshteinSnomed(wordstrOrig.lower(), termDict, config)
            if ldOrig:
                snomedNumber, el = ldOrig 
                return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, snomedNumber, name, combineCategories)
        
        if searchedTermLength <= config.maxLengthForPermutations and searchedTermLength >= config.minLengthForPermutations and config.doPermutations:
            perms = permutations.getPermutations(wordstrOrig)
            for el in perms:
                if termDict.has_key(el.lower()):
                    print "*" + el.lower()
                    snomedNumber = termDict[el.lower()]
                    return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, snomedNumber, name, combineCategories)
            perms = permutations.getPermutations(wordstrLemma)
            for el in perms:
                if termDict.has_key(el.lower()):
                    snomedNumber = termDict[el.lower()]
                    print "*" + el.lower()
                    return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, snomedNumber, name, combineCategories)
        if config.doCompoundSplitting and searchedTermLength == 1 and len(wordstrOrig) >= config.minLengthForCompoundSplitting:
            print wordstrOrig
            compoundMatchLemma = getCompoundMatch(wordstrLemma, termDict, config)
            if compoundMatchLemma:
                print compoundMatchLemma
                return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, compoundMatchLemma, name, combineCategories)
            if wordstrOrig != wordstrLemma:
                compoundMatchOrig = getCompoundMatch(wordstrOrig, termDict, config)
                if compoundMatchOrig:
                    return getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, compoundMatchOrig, name, combineCategories)
            
            
            
    #print "inget hittat length", wordList
    return 0 #nothing found                   

"""
#TODO: Detta är en kombinerad. Förmodligen är det bättre att göra enligt den nedan, att ta det sista ordet bara.
def getCompoundMatch(wordstrOrig, termDict):
     split =  compoundSplitting.invokeSplitter(wordstrOrig)
     found = []
     if split:
         for el in split:
             if termDict.has_key(el):
                 snomedNumber = termDict[el]
                 if snomedNumber not in found:
                     found.append(snomedNumber)
     if found:
         return "_".join(found)
     else:
         return None
"""

def getCompoundMatch(wordstrOrig, termDict, config):
     split = compoundSplitting.invokeSplitter(wordstrOrig)
     time.sleep(1)
     found = None
     if split:
         print split
         for el in split:
             ret = getLevenshteinSnomed(el.lower(), termDict, config)
             if ret != []: 
                 found = ret[0]
             print found
     return found
    
def getLevenshteinSnomed(word, termDic, config):
    if len(word) >= config.maxLengthForLevenshtein or len(word) <= config.minLengthForLevenshtein:
        return []
    ldList = levenshtein.generateLDOneWords(word)
    for el in ldList:
        if termDict.has_key(el):
            snomedNumber = termDict[el]
            return snomedNumber, el
    return []

def getFeatureListWhenATermIsFound(searchedTermLength, wordListOrig, wordListLemma, termDict, index, snomedNumber, name, combineCategories):
    beforeOrig = wordListOrig[0: index]
    afterOrig = wordListOrig[index + searchedTermLength:len(wordListOrig)]
    beforeLemma = wordListLemma[0: index]
    afterLemma = wordListLemma[index + searchedTermLength:len(wordListLemma)]
    afterresult = findTermInSequence(afterOrig, afterLemma, termDict, name, combineCategories)
    beforeresult = findTermInSequence(beforeOrig, beforeLemma, termDict, name, combineCategories)
    totMoreResult = []
    if(searchedTermLength > 1):
        firstWordInMatchOrig = wordListOrig[index:index+1]
        restOfMatchOrig = wordListOrig[index+1:index + searchedTermLength]
        firstWordInMatchLemma = wordListLemma[index:index+1]
        restOfMatchLemma = wordListLemma[index+1:index + searchedTermLength]
        firstMoreResult = findTermInSequence(firstWordInMatchOrig, firstWordInMatchLemma, termDict, name, combineCategories)
        restMoreResult = findTermInSequence(restOfMatchOrig, restOfMatchLemma, termDict, name, combineCategories)
        totMoreResult = firstMoreResult + restMoreResult
        #print "rest" + str(totMoreResult)
    #currentresultMore = 
    currentresult = []
    if totMoreResult == []:
        for el in wordListOrig[index:index + searchedTermLength]:
            currentresult.append((el, snomedNumber))
    else:
        for el, (word, moreSnomed) in zip(wordListOrig[index:index + searchedTermLength],totMoreResult):
            toAppend = ""
            if moreSnomed != snomedNumber and moreSnomed != NOMATCH and combineCategories:
                toAppend = "_" + moreSnomed
            currentresult.append((el, snomedNumber + toAppend))
    resultList = beforeresult + currentresult + afterresult
    return resultList

def doSnomedFeatureFile(inputfile, outputfile, name, combineCategories, reversed):
    config = common.getImport(name, name + "_config")
    termDict = createDictionary(combineCategories, name)
    outputfile = open(outputfile, "w")
    granskaFeatureFile = open(inputfile, "r")
    rowReader = csv.reader(granskaFeatureFile, delimiter='\t', quotechar='|')
    words = []
    print "Starts searching for terminology matches"
    nrOfSentences = 0
    for row in rowReader:
        if row == []:
            nrOfSentences = nrOfSentences + 1
            writeFeatures(words, termDict, name, outputfile, reversed, combineCategories)
            words = []
        else:
            words.append(row)
    if words != []:
        writeFeatures(words, termDict, name, outputfile, reversed, combineCategories)
    outputfile.close()
    granskaFeatureFile.close()
    print "Ready with terminology matches"

def writeFeatures(words, termDict, name, outputfile, reversed, combineCategories):
    result = findTermInMatrix([[common.getUnicodedWordFromUTFCoded(word) for word in wordlist] for wordlist in words], termDict, name, combineCategories)
    lastResult = NOMATCH
    for (lemma, snomed), word in zip(result, words):
        if not reversed:
            listToWrite = word[0:len(word)-1] + [snomed] + [word[len(word)-1]]
        else: # if it is configured to rule-based evaluation
            prefix = ""
            listToWrite = word[0:len(word)-1] + [word[len(word)-1]] + [prefix + snomed]
            lastResult = snomed
        for el,nr in zip(listToWrite, range(0, len(listToWrite)-1)):
            outputfile.write(el + '\t')
        outputfile.write(listToWrite[len(listToWrite)-1] + '\n')     
    outputfile.write('\n')



