# -*- coding: utf-8 -*-

import runCRFProperties
import os
import doSnomedFeatureFileFromGranskaFeatures
import runCRFProperties
import common
import foldEvaluationConfig.configForSerie
import doParsing

# Also removes all html-tags in the text

NOTAG = u"O"
FEATURES_DELIM = u"\t"


def replaceWhiteSpace(str):
    str = str.replace(' ', '_')
    str = str.replace('\t', '_')
    return str

def getPrefix(str, nrOfChars):
    return str[0:nrOfChars]

def getSuffix(str, nrOfChars):
    return str[len(str):-nrOfChars-1:-1]

def generateFeatures(inputfilename, outputfilename, name, combineCategories, reversed):
    print "Starts to generate features from tags"
    common.ensure_dir(runCRFProperties.featuresTempDir)
    output = []
    currenttag = NOTAG
    isFirstTokenInTagSequence = 1
    inpFile = open(inputfilename)
    text = common.getUnicodedWordFromUTFCoded(inpFile.read())
    inpFile.close()
    print "Encoding ready"
    text = text.replace("\n", " ")
    text = text.replace("\t", " ")
    text = text.replace("< / ", "</")
    index = 0
    while index < len(text):
        if text[index] == "<":
            if index > 0:
                if text[index - 1] != ' ':
                    output.append((' ', currenttag))
            currenttag, index, isFirstTokenInTagSequence = getTag(text, index, currenttag)
        else:
            currentChar = text[index]
            output.append((currentChar, currenttag))
            index = index + 1

    outputfile = "/processedGranskaFeatures.txt"
    print "run Granske"
    doGranska(output, outputfile)
    print "Use terminology lists"

    config = common.getImport(name, name + "_config")
    if config.doMaltParsing:
        maltOutputfile = "/maltOutput.txt"
        doMaltParser(outputfile, maltOutputfile, runCRFProperties.featuresTempDir, name)

    doSnomedFeatureFileFromGranskaFeatures.doSnomedFeatureFile(runCRFProperties.featuresTempDir + outputfile, outputfilename, name, combineCategories, reversed)

def doMaltParser(outputfile, maltOutputfile, dir, name):
    doParsing.parse(outputfile, maltOutputfile, dir, name)

def doEmulatedGranska():
    pass

def doGranska(tagProcessOutput, outputfile):
    granskaoutput, latincodedoutput = executeGranska(tagProcessOutput)
    granskaprocessedOutput = processGranskaOutput(granskaoutput, latincodedoutput)

    features = open(runCRFProperties.featuresTempDir + outputfile, "w")
    for el in granskaprocessedOutput:
        features.write(common.getUTFCodedWord(el + u'\n'))
    features.close()


def processGranskaOutput(granskaoutput, wordtuplelist):
    test = open(runCRFProperties.featuresTempDir + "/processedText.txt", "w")
    wordSeparators = open(runCRFProperties.featuresTempDir + "/wordSeparators.txt", "w")

    for el in wordtuplelist:
        test.write(el[0] + '\n')

    wordtuplelistIndex = 0
    granskaprocessedOutput = []
    currentTag = NOTAG
    for line in granskaoutput:
        prefix = None
        pos, posfirst, word, lemma =  getGranskaOutputLine(line)
        for currentChar in word:
            currentWordtuplelistIndex = wordtuplelistIndex
            try:
                while not (currentChar.lower() == wordtuplelist[wordtuplelistIndex][0].lower()):
                    #print "Fel " + currentChar + "*" + wordtuplelist[wordtuplelistIndex][0] + "*"
                    wordSeparators.write("*" + wordtuplelist[wordtuplelistIndex][0]+ "*")
                    wordtuplelistIndex = wordtuplelistIndex + 1
                #print currentChar,  wordtuplelist[wordtuplelistIndex][0]
            except IndexError:
                print "Error in mathcing granska with annotations at index " +  str(currentWordtuplelistIndex) \
                    + " (char: " + currentChar + ")"
                exit(1)
            if prefix == None:
                if wordtuplelist[wordtuplelistIndex][1] == NOTAG:
                    prefix = u""
                elif wordtuplelist[wordtuplelistIndex][1] != currentTag:
                    prefix = u"B-"
                else: 
                    prefix = u"I-"
                currentTag = wordtuplelist[wordtuplelistIndex][1]
            wordtuplelistIndex = wordtuplelistIndex + 1
        granskaprocessedOutput.append(common.latinDecode(replaceWhiteSpace(word)) + u'\t'\
                                          + common.latinDecode(posfirst) + u'\t' \
                                          + common.latinDecode(pos) + u'\t' \
                                          + common.latinDecode(replaceWhiteSpace(lemma)) + u'\t' \
                                          + getPrefix(common.latinDecode(replaceWhiteSpace(lemma)), 4) + u'\t' \
                                          + getSuffix(common.latinDecode(replaceWhiteSpace(lemma)), 4) + u'\t' \
                                          + prefix + common.latinDecode(currentTag))
        #granskaprocessedOutput.append(latinDecode(word) + u'\t' + latinDecode(pos) \
         #                                 + u'\t' + prefix + latinDecode(currentTag))
        if posfirst == "mad":
            granskaprocessedOutput.append(u'')
    return granskaprocessedOutput
 
def getTag(text, index, currentTag):
    index = index + 1
    tag = ""
    startTag = 1
    if text[index] == "/":
        startTag = 0
        index = index + 1
    while index < len(text) and text[index] != ">":
        tag = tag + text[index]
        index = index + 1
    if startTag and tag in foldEvaluationConfig.configForSerie.tagsToEvaluate:
        return tag, index + 1, 1
    elif not startTag and tag == currentTag:
        return NOTAG, index + 1, 0
    else:
        return currentTag, index + 1, 0

def getToken (text, index):
    token = ""
    while index < len(text) and not isSeparatorAtIndex(text, index) and text[index] != "<":
        token = token + text[index]
        index = index + 1
    if index >= len(text) or isSeparatorAtIndex(text, index):
        return token, index + 1
    else:
        return token, index # then it is '<'

def executeGranska(wordtuplelist):
    GRANSKA_OUTPUT = runCRFProperties.featuresTempDir + "/granskaoutput.txt"
    GRANSKA_INPUT_FILE =  runCRFProperties.featuresTempDir + "/granskainput.txt"
    granskaoutput = []
    latincodedoutput = []

    out = open(GRANSKA_INPUT_FILE, "w")
    for el in wordtuplelist:
        encoded = None
        try:
            encoded = el[0].encode('ISO 8859-1')
        except UnicodeEncodeError:
            try:
                print "Failed to decode " + el[0] + ". Will be replaced by 'a'"
            except:
                 print "Failed to decode character. Will be replaced by 'a'"
            encoded ='a'.encode('ISO 8859-1')
        out.write(encoded)
        latincodedoutput.append((encoded, el[1]))
    out.close()
    
    execCommand = foldEvaluationConfig.configForSerie.granskaPath + "tagg -IBa -l " \
        + foldEvaluationConfig.configForSerie.granskaPath + "lex/ " + GRANSKA_INPUT_FILE + " > " + GRANSKA_OUTPUT
    os.system(execCommand)
    granskaoutput = open(GRANSKA_OUTPUT).readlines()
    return granskaoutput, latincodedoutput

def getGranskaOutputLine(line):
    try:
        splitted = line.split('\t')
    except:
        splitted = line.split('\t')
        print "Could not decode"
    word = splitted[0]
    pos = splitted[1]
    posfirst = pos.split(".")[0]
    lemma = splitted[2]
    return pos, posfirst, word, lemma


  
#if __name__ == "__main__":
#    generateFeatures("testfil.txt", "features.txt", combineCategories = True, reversed = False)
   
