from string import Template
import runCRFProperties
import os
import foldEvaluationConfig.configForSerie
from decimal import Decimal
import common

import sys
sys.path.append("./foldEvaluationConfig")

DECIMALPL = 2

def getNumberStrInLine(line, textToMatch, tag):
    if line.startswith(tag + textToMatch):
        value = line.replace(tag + textToMatch, "").strip()
        value = value.replace(")", "")
        if value != common.UNDEFINED:
            value = str(round(Decimal(value), DECIMALPL))
        return value
    else:
        #print "inget hittat"
        return 0

texDocument =  Template(\

"""
\\documentclass[11pt]{article}
\\author{Maria Skeppstedt}
\\title{Results from applied rules}
\\usepackage{verbatim}
\\usepackage[T1]{fontenc}
\\usepackage[utf8x]{inputenc}
\\addtolength{\\oddsidemargin}{-.5in}
\\addtolength{\\evensidemargin}{-.5in}
\\addtolength{\\textwidth}{1in}
\\usepackage{float}
\\author{Maria Skeppstedt}

%%% BEGIN DOCUMENT
\\begin{document}

\\maketitle
$serieDescription



\\subsection*{Descriptions of each test run}
In the test serie, the following tests were run:

$descriptionlist

$tablesections
\\end{document}
"""\
)

tableTemplate = Template(\
"""
\\subsection*{$title}

\\begin{table}[H]
   \\centering

   \\begin{tabular}{@{} |l|rrr| @{}} 
\hline
      Nr.    & Precision (CI) & Recall (CI) & F-Score\\\\ \\hline \hline
      $moretablerows
\hline
   \\end{tabular}
   \\caption{Results for the tag $title}
   \\label{tab:booktabs}
\\end{table}
"""\
)

descriptionTemplate = Template(\
"""
\\begin{description}
\item[$name] $description 
\end{description}
"""\
)

table = ""
subdirs = filter(lambda x: x != runCRFProperties.outputFromSplitFile, os.walk(runCRFProperties.resultsAllRuns).next()[1])

for tag in foldEvaluationConfig.configForSerie.tagsToEvaluate:
    texline = "" 
   
    for dir in subdirs:
        texline = texline + dir.replace("_", "\\_") + " & "
        resultsFile = open(runCRFProperties.resultsAllRuns + "/" + dir + "/" + runCRFProperties.crfoutputfile)
        for line in resultsFile:
            value = getNumberStrInLine(line, ": mean altprecision, all folds:", tag)
            if value: 
                texline = texline + value
            value = getNumberStrInLine(line, ": ( 95-conf altprecision, all folds: ", tag)
            if value:
                texline = texline + " ($\\pm$ " + value  + ") & "
            value = getNumberStrInLine(line, ": mean altrecall, all folds:", tag)
            if value:
                texline = texline + value
            value = getNumberStrInLine(line, ": ( 95-conf altrecall, all folds: ", tag)
            if value:
                texline = texline + " ($\\pm$ " + value  + ") & "
            value = getNumberStrInLine(line, ": mean altfscore, all folds:", tag)
            if value:
                texline = texline + value  + " \\\\ \n" 
        resultsFile.close()
        
    table = table + tableTemplate.substitute(\
        title = tag.replace("_", "\\_"), \
            moretablerows = texline \
        ) + '\n'

summaryForRun = ""
descriptionText = ""
for dir in subdirs:
    summaryForRun = summaryForRun + dir.replace("_", "\\_") + " & "
    resultsFile = open(runCRFProperties.resultsAllRuns + "/" + dir + "/" + runCRFProperties.crfoutputfile)
    for line in resultsFile:
         value = getNumberStrInLine(line, "Mean altprecision for all tags: ", "")
         if value:
             summaryForRun = summaryForRun + value + " & "
         value = getNumberStrInLine(line, "Mean altrecall for all tags: ", "")
         if value:
             summaryForRun = summaryForRun + value + " & "
         value = getNumberStrInLine(line, "Mean altfscore for all tags: ", "")
         if value:
             summaryForRun = summaryForRun + value + " \\\\ \n"
    resultsFile.close()

    config = common.getImport(dir, dir + "_config")
    descriptionText = descriptionText + descriptionTemplate.substitute(\
        name = dir.replace("_", "\\_"), \
            description = config.description \
        ) + '\n'

table = table + tableTemplate.substitute(\
    title = "Mean for all tags", \
        moretablerows = summaryForRun \
        ) + '\n'

finalDocument = texDocument.substitute(\
                       tablesections = table, \
                       descriptionlist = descriptionText, \
                       serieDescription = foldEvaluationConfig.configForSerie.serieDescription \

                           )

texfile = open(runCRFProperties.resultsAllRuns + "/resultsInTex.tex", 'w')
texfile.write(finalDocument)
texfile.close()
